@@ -29,6 +29,10 @@ from .proto.cosmos.authz.v1beta1 import (
     query_pb2 as authz_query,
     authz_pb2 as authz_type,
 )
+from .proto.cosmos.bank.v1beta1 import (
+    query_pb2_grpc as bank_query_grpc,
+    query_pb2 as bank_query,
+)
 from .proto.cosmos.tx.v1beta1 import (
     service_pb2_grpc as tx_service_grpc,
     service_pb2 as tx_service,
@@ -58,6 +62,11 @@ DEFAULT_TIMEOUTHEIGHT_SYNC_INTERVAL = 10 # seconds
 DEFAULT_TIMEOUTHEIGHT = 20 # blocks
 DEFAULT_SESSION_RENEWAL_OFFSET = 120 # seconds
 DEFAULT_BLOCK_TIME = 3 # seconds
+DEFAULT_CHAIN_COOKIE_NAME = '.chain_cookie'
+
+# use append mode to create file if not exist
+cookie_file = open(DEFAULT_CHAIN_COOKIE_NAME, "a+")
+cookie_file.close()
 
 class AsyncClient:
     def __init__(
@@ -87,8 +96,15 @@ class AsyncClient:
         self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(self.chain_channel)
         self.stubAuth = auth_query_grpc.QueryStub(self.chain_channel)
         self.stubAuthz = authz_query_grpc.QueryStub(self.chain_channel)
+        self.stubBank = bank_query_grpc.QueryStub(self.chain_channel)
         self.stubTx = tx_service_grpc.ServiceStub(self.chain_channel)
-        self.chain_cookie = ""
+
+        # attempt to load from disk
+        cookie_file = open(DEFAULT_CHAIN_COOKIE_NAME, "r+")
+        self.chain_cookie = cookie_file.read()
+        cookie_file.close()
+        print("chain session cookie loaded from disk:", self.chain_cookie)
+
         self.exchange_cookie = ""
         self.timeout_height = 1
 
@@ -205,7 +221,14 @@ class AsyncClient:
             return
 
         if type == "chain":
+            # write to client instance
             self.chain_cookie = new_cookie
+            # write to disk
+            cookie_file = open(DEFAULT_CHAIN_COOKIE_NAME, "w")
+            cookie_file.write(new_cookie)
+            cookie_file.close()
+            print("chain session cookie saved to disk")
+
         if type == "exchange":
             self.exchange_cookie = new_cookie
 
@@ -286,6 +309,21 @@ class AsyncClient:
             )
         )
 
+    async def get_bank_balances(self, address: str):
+        return await self.stubBank.AllBalances(
+            bank_query.QueryAllBalancesRequest(
+                address=address
+            )
+        )
+
+    async def get_bank_balance(self, address: str, denom: str):
+        return await self.stubBank.Balance(
+            bank_query.QueryBalanceRequest(
+                address=address,
+                denom=denom
+            )
+        )
+
     # Injective Exchange client methods
 
     # Auction RPC
@@ -328,6 +366,78 @@ class AsyncClient:
         req = explorer_rpc_pb.GetTxByTxHashRequest(hash=tx_hash)
         return await self.stubExplorer.GetTxByTxHash(req)
 
+    async def get_account_txs(self, address: str, **kwargs):
+        req = explorer_rpc_pb.GetAccountTxsRequest(
+            address=address,
+            before=kwargs.get("before"),
+            after=kwargs.get("after"),
+            limit=kwargs.get("limit"),
+            skip=kwargs.get("skip"),
+            type=kwargs.get("type"),
+            module=kwargs.get("module"))
+        return await self.stubExplorer.GetAccountTxs(req)
+
+    async def get_blocks(self, **kwargs):
+        req = explorer_rpc_pb.GetBlocksRequest(
+            before=kwargs.get("before"),
+            after=kwargs.get("after"),
+            limit=kwargs.get("limit")
+            )
+        return await self.stubExplorer.GetBlocks(req)
+
+    async def get_block(self, block_height: str):
+        req = explorer_rpc_pb.GetBlockRequest(id=block_height)
+        return await self.stubExplorer.GetBlock(req)
+
+    async def get_txs(self, **kwargs):
+        req = explorer_rpc_pb.GetTxsRequest(
+            before=kwargs.get("before"),
+            after=kwargs.get("after"),
+            limit=kwargs.get("limit"),
+            skip=kwargs.get("skip"),
+            type=kwargs.get("type"),
+            module=kwargs.get("module"))
+        return await self.stubExplorer.GetTxs(req)
+
+    async def stream_txs(self):
+        req = explorer_rpc_pb.StreamTxsRequest()
+        return self.stubExplorer.StreamTxs(req)
+
+    async def stream_blocks(self):
+        req = explorer_rpc_pb.StreamBlocksRequest()
+        return self.stubExplorer.StreamBlocks(req)
+
+    async def get_peggy_deposits(self, **kwargs):
+        req = explorer_rpc_pb.GetPeggyDepositTxsRequest(
+            sender=kwargs.get("sender"),
+            receiver=kwargs.get("receiver"),
+            limit=kwargs.get("limit"),
+            skip=kwargs.get("skip")
+        )
+        return await self.stubExplorer.GetPeggyDepositTxs(req)
+
+    async def get_peggy_withdrawals(self, **kwargs):
+        req = explorer_rpc_pb.GetPeggyWithdrawalTxsRequest(
+            sender=kwargs.get("sender"),
+            receiver=kwargs.get("receiver"),
+            limit=kwargs.get("limit"),
+            skip=kwargs.get("skip")
+        )
+        return await self.stubExplorer.GetPeggyWithdrawalTxs(req)
+
+    async def get_ibc_transfers(self, **kwargs):
+        req = explorer_rpc_pb.GetIBCTransferTxsRequest(
+            sender=kwargs.get("sender"),
+            receiver=kwargs.get("receiver"),
+            src_channel=kwargs.get("src_channel"),
+            src_port=kwargs.get("src_port"),
+            dest_channel=kwargs.get("dest_channel"),
+            dest_port=kwargs.get("dest_port"),
+            limit=kwargs.get("limit"),
+            skip=kwargs.get("skip")
+        )
+        return await self.stubExplorer.GetIBCTransferTxs(req)
+
     # AccountsRPC
 
     async def stream_subaccount_balance(self, subaccount_id: str):
@@ -359,6 +469,8 @@ class AsyncClient:
             subaccount_id=subaccount_id,
             denom=kwargs.get("denom"),
             transfer_types=kwargs.get("transfer_types"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubExchangeAccount.SubaccountHistory(req)
 
@@ -462,6 +574,8 @@ class AsyncClient:
             market_id=market_id,
             order_side=kwargs.get("order_side"),
             subaccount_id=kwargs.get("subaccount_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubSpotExchange.Orders(req)
 
@@ -511,7 +625,10 @@ class AsyncClient:
 
     async def get_spot_subaccount_orders(self, subaccount_id: str, **kwargs):
         req = spot_exchange_rpc_pb.SubaccountOrdersListRequest(
-            subaccount_id=subaccount_id, market_id=kwargs.get("market_id")
+            subaccount_id=subaccount_id,
+            market_id=kwargs.get("market_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubSpotExchange.SubaccountOrdersList(req)
 
@@ -521,6 +638,8 @@ class AsyncClient:
             market_id=kwargs.get("market_id"),
             execution_type=kwargs.get("execution_type"),
             direction=kwargs.get("direction"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubSpotExchange.SubaccountTradesList(req)
 
@@ -555,6 +674,8 @@ class AsyncClient:
             market_id=market_id,
             order_side=kwargs.get("order_side"),
             subaccount_id=kwargs.get("subaccount_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubDerivativeExchange.Orders(req)
 
@@ -604,7 +725,10 @@ class AsyncClient:
 
     async def get_derivative_positions(self, market_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.PositionsRequest(
-            market_id=market_id, subaccount_id=kwargs.get("subaccount_id")
+            market_id=market_id,
+            subaccount_id=kwargs.get("subaccount_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubDerivativeExchange.Positions(req)
 
@@ -620,13 +744,18 @@ class AsyncClient:
 
     async def get_derivative_liquidable_positions(self, **kwargs):
         req = derivative_exchange_rpc_pb.LiquidablePositionsRequest(
-            market_id=kwargs.get("market_id")
+            market_id=kwargs.get("market_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubDerivativeExchange.LiquidablePositions(req)
 
     async def get_derivative_subaccount_orders(self, subaccount_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.SubaccountOrdersListRequest(
-            subaccount_id=subaccount_id, market_id=kwargs.get("market_id")
+            subaccount_id=subaccount_id,
+            market_id=kwargs.get("market_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubDerivativeExchange.SubaccountOrdersList(req)
 
@@ -636,6 +765,8 @@ class AsyncClient:
             market_id=kwargs.get("market_id"),
             execution_type=kwargs.get("execution_type"),
             direction=kwargs.get("direction"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit")
         )
         return await self.stubDerivativeExchange.SubaccountTradesList(req)
 
