@@ -112,7 +112,7 @@ class Network:
         )
 
     @classmethod
-    def mainnet(cls, node='lb'):
+    def mainnet(cls, node='k8s'):
         nodes = [
             'k8s',
             'lb',
@@ -126,9 +126,9 @@ class Network:
             raise ValueError('Must be one of {}'.format(nodes))
 
         if node == 'lb' or node == 'k8s':
-            lcd_endpoint=f'https://{node}.mainnet.lcd.injective.network'
-            grpc_endpoint=f'{node}.mainnet.chain.grpc.injective.network:443'
-            grpc_exchange_endpoint=f'{node}.mainnet.exchange.grpc.injective.network:443'
+            lcd_endpoint='https://k8s.mainnet.lcd.injective.network'
+            grpc_endpoint='k8s.mainnet.chain.grpc.injective.network:443'
+            grpc_exchange_endpoint='k8s.mainnet.exchange.grpc.injective.network:443'
         else:
             lcd_endpoint='https://lcd.injective.network'
             grpc_endpoint=f'{node}.injective.network:9900'
